/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.ProjectLink;
import ch.hortis.sonar.mvn.CoreMojo;
import ch.hortis.sonar.mvn.Report;
import org.apache.maven.model.CiManagement;
import org.apache.maven.model.IssueManagement;
import org.apache.maven.model.Scm;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.picocontainer.DefaultPicoContainer;
import org.picocontainer.behaviors.Caching;
import org.picocontainer.lifecycle.ReflectionLifecycleStrategy;
import org.picocontainer.monitors.NullComponentMonitor;
import org.sonar.commons.ReadOnlyRulesRepository;
import org.sonar.commons.SaveOnDemandMetricsRepository;
import org.sonar.commons.database.BatchDatabaseManager;
import org.sonar.maven.sources.SourcesCollector;

import java.util.Date;
import java.util.List;

/**
 * Update the project definition and collect measures
 *
 * @goal collect
 */
public class CollectMojo extends CoreMojo {

  public void doExecute() throws MojoExecutionException, MojoFailureException {
    // components are started/stopped by reflection. It executed the following methods :
    ReflectionLifecycleStrategy lifecycleStrategy = new ReflectionLifecycleStrategy(new
        NullComponentMonitor(), "start", "stop", "dispose");

    // one instance per component
    DefaultPicoContainer pico = new DefaultPicoContainer(new Caching(), lifecycleStrategy, null);
    try {
      registerComponents(pico);
      pico.start();


      // create or update the project
      MeasuresRecorderImpl recorder = pico.getComponent(MeasuresRecorderImpl.class);
      updateProject(recorder.getRoot(), getProject());
      recorder.persistRoot();

      // collect measures and sources
      if (!getProject().getPackaging().equals("pom")) {
        for (Collector collector : pico.getComponents(Collector.class)) {
          getLog().debug("collecting " + collector.getClass().getSimpleName());
          collector.collect();
        }
      }

      setSnapshotId(recorder.getRootSnapshot().getId(), getProject());

    } finally {
      try {
        pico.stop();
      } catch (Exception e) {
        getLog().warn(e);
      }
    }
  }

  private void registerComponents(DefaultPicoContainer pico) {
    pico.addComponent(new Date());
    pico.addComponent(MavenProject.newMavenProject(getProject().getGroupId(), getProject().getArtifactId(), getBranch()));
    pico.addComponent(SaveOnDemandMetricsRepository.class);
    pico.addComponent(ReadOnlyRulesRepository.class);
    pico.addComponent(getProject());
    pico.addComponent(MeasuresRecorderImpl.class);
    pico.addComponent(getDatabaseConnector());
    pico.addComponent(BatchDatabaseManager.class);

    for (Report report : getTargetReports()) {
      if (report.getCollectorClass() != null) {
        pico.addComponent(report.getCollectorClass());
      }
    }
    pico.addComponent(SourcesCollector.class);
  }

  private List<Report> getTargetReports() {
    if (isLight()) {
      return Report.getLightReports();
    }
    return Report.getReports();
  }

  private void updateProject(MavenProject sonarProject, org.apache.maven.project.MavenProject mavenProject) {
    sonarProject.setName(mavenProject.getName());
    sonarProject.setDescription(mavenProject.getDescription());
    updateProjectLink(ProjectLink.LINK_HOME_PAGE, mavenProject.getUrl(), sonarProject);
    Scm scmConfig = mavenProject.getScm();
    if (scmConfig == null) {
      scmConfig = new Scm();
    }
    updateProjectLink(ProjectLink.LINK_SCM_URL, scmConfig.getUrl(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_DEV_CONNECTION, scmConfig.getDeveloperConnection(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_RO_CONNECTION, scmConfig.getConnection(), sonarProject);

    CiManagement cimConfig = mavenProject.getCiManagement();
    if (cimConfig == null) {
      cimConfig = new CiManagement();
    }
    updateProjectLink(ProjectLink.LINK_CONTINUOUS_INTEGRATION, cimConfig.getUrl(), sonarProject);

    IssueManagement imConfig = mavenProject.getIssueManagement();
    if (imConfig == null) {
      imConfig = new IssueManagement();
    }
    updateProjectLink(ProjectLink.LINK_ISSUES_TRACKER, imConfig.getUrl(), sonarProject);
  }

  private void updateProjectLink(String linkType, String href, MavenProject sonarProject) {
    if (href != null && !"".equals(href)) {
      ProjectLink link = sonarProject.getProjectLinkByType(linkType);
      if (link == null) {
        link = new ProjectLink();
        link.setMavenProject(sonarProject);
        link.setType(linkType);
        sonarProject.getProjectLinks().add(link);
      }
      link.setHref(href);
    }
  }
}
