/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.clover;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.mvn.PomUtils;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.maven.project.MavenProject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonar.commons.MetricsRepository;
import org.sonar.maven.AbstractCollector;
import org.sonar.maven.MeasuresRecorder;
import org.sonar.maven.XmlParserException;
import org.sonar.maven.XmlReportParser;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import java.io.File;
import java.text.ParseException;

public class CloverCollector extends AbstractCollector {

  private XmlReportParser parser;
  private MeasuresRecorder recorder;
  private static final Logger LOG = LoggerFactory.getLogger(CloverCollector.class);

  public CloverCollector(MavenProject project, MeasuresRecorder recorder) {
    File reportFile = findFileFromBuildDirectory(project, "site/clover/clover.xml");
    if (reportFile != null) {
      parser = new XmlReportParser();
      parser.parse(reportFile);
      this.recorder = recorder;
    }
  }

  public CloverCollector(File xmlFile, MeasuresRecorder recorder) {
    parser = new XmlReportParser();
    parser.parse(xmlFile);
    this.recorder = recorder;
  }

  public Metric getMetric() {
    return MetricsRepository.CODE_COVERAGE;
  }

  public void collect() {
    try {
      if (parser != null) {
        collectProjectMeasures();
        collectPackageMeasures();
        collectFileMeasures();
      }

    } catch (ParseException e) {
      throw new XmlParserException(e);

    }
  }

  protected void collectProjectMeasures() throws ParseException {
    Double result = null;
    try {
      Element projectEl = parser.getChildElement(parser.getRoot(), "project");
      Element metricsEl = parser.getChildElement(projectEl, "metrics");
      result = getCodeCoverageFromMetricsNode(metricsEl);
      recorder.createProjectMeasure(new MeasureKey(getMetric()), result);
    } catch (IllegalArgumentException e) {
      LOG.warn("result is invalid : ", result);
    }
  }

  private void collectPackageMeasures() throws ParseException {
    Double result = null;
    try {
      NodeList packages = parser.executeXPathNodeList("/coverage/project/package");
      for (int i = 0; i < packages.getLength(); i++) {
        Element pkElt = (Element) packages.item(i);
        Element metricsEl = parser.getChildElement(pkElt, "metrics");
        result = getCodeCoverageFromMetricsNode(metricsEl);
        recorder.createPackageMeasure(pkElt.getAttribute("name"), new MeasureKey(getMetric()), result);
      }
    } catch (IllegalArgumentException e) {
      LOG.warn("result is invalid : ", result);
    }
  }

  private void collectFileMeasures() throws ParseException {
    Double result = null;
    try {
      NodeList packages = parser.executeXPathNodeList("/coverage/project/package");
      for (int i = 0; i < packages.getLength(); i++) {
        Element pkElt = (Element) packages.item(i);
        String namespace = pkElt.getAttribute("name");
        NodeList files = parser.executeXPathNodeList(pkElt, "file");
        for (int j = 0; j < files.getLength(); j++) {
          Element fileElt = (Element) files.item(j);
          String filename = PomUtils.extractClassName(fileElt);

          Element metricsElt = (Element) parser.executeXPath(fileElt, XPathConstants.NODE, "metrics");
          if (canBeIncludedInFileMetrics(metricsElt)) {
            result = getCodeCoverageFromMetricsNode(metricsElt);
            recorder.createClassMeasure(namespace, filename, false, new MeasureKey(getMetric()), result);
          }
        }
      }
    } catch (IllegalArgumentException e) {
      LOG.warn("result is invalid : ", result);
    }
  }

  private boolean canBeIncludedInFileMetrics(Element metricsNode) throws ParseException {
    return parseNumber(metricsNode.getAttribute("elements")) > 0;
  }

  private Double getCodeCoverageFromMetricsNode(Element metricsNode) throws ParseException {
    double coveredElements = parseNumber(metricsNode.getAttribute("coveredelements"));
    double nb = parseNumber(metricsNode.getAttribute("elements"));
    return 100.0 * (coveredElements / nb);
  }

}
