/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.cpd;

import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.project.MavenProject;
import org.sonar.commons.MetricsRepository;
import org.sonar.maven.*;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.*;

public class CpdCollector extends AbstractCollector {
  private XmlReportParser parser;
  private MeasuresRecorder recorder;
  private List<String> sourceDirs;

  public CpdCollector(MavenProject project, MeasuresRecorder recorder) {
    File xmlFile = findFileFromBuildDirectory(project, "cpd.xml");
    if (xmlFile != null && xmlFile.exists()) {
      parser = new XmlReportParser();
      parser.parse(readXmlWithoutEncodingErrors(xmlFile));
      sourceDirs = project.getCompileSourceRoots();
      this.recorder = recorder;
    }
  }

  /**
   * only for unit tests
   */
  public CpdCollector(File xmlFile, String[] sourceDirs, MeasuresRecorder recorder) {
    parser = new XmlReportParser();
    parser.parse(readXmlWithoutEncodingErrors(xmlFile));
    this.sourceDirs = Arrays.asList(sourceDirs);
    this.recorder = recorder;
  }


  /**
   * only for unit tests
   */
  protected CpdCollector() {
  }

  public void collect() {
    try {
	  if (parser != null) {
        collectFileMeasures();
      }

    } catch (ParseException e) {
      throw new XmlParserException(e);
    }
  }

  private void collectFileMeasures() throws ParseException {
    NodeList duplications = parser.executeXPathNodeList("/pmd-cpd/duplication");
    Map<String, ClassTempData> classData = new HashMap<String, ClassTempData>();
    for (int i=0; i < duplications.getLength(); i++) {
      Element duplication = (Element) duplications.item(i);

      NodeList files = parser.executeXPathNodeList(duplication, "file");
      processFileMeasure(classData, (Element) files.item(0), duplication);
      processFileMeasure(classData, (Element) files.item(1), duplication);
    }

    for (ClassTempData data : classData.values()) {
      data.saveUsing(recorder);
    }
  }

  private String readXmlWithoutEncodingErrors(File file) {
    try {
      // First step : the file is read with system charset encoding. It should resolve the problem in most cases
      String xml = FileUtils.readFileToString(file);

      // second step : remove CDATA nodes that contain wrong characters. Those nodes are not needed by the collector.
      return removeCDataNodes(xml);

    } catch (IOException e) {
      throw new XmlParserException("can not read the file " + file.getAbsolutePath(), e);
    }
  }

  String removeCDataNodes(String xml) {
    String result = xml;
    String startNode = "<codefragment>";
    String endNode = "</codefragment>";
    String[] subs = StringUtils.substringsBetween(xml, startNode, endNode);
    if (subs != null) {
      for (String sub : subs) {
        result = StringUtils.remove(result, startNode + sub + endNode);
      }
    }
    return result;
  }

  private void processFileMeasure(Map<String, ClassTempData> fileContainer, Element fileEl, Element duplication) throws ParseException {
    String filePath = fileEl.getAttribute("path").replace('\\', '/');
    String fileName = StringUtils.substringAfterLast(filePath, "/");
    fileName = StringUtils.substringBeforeLast(fileName, ".java");
    String packageName = StringUtils.substringBeforeLast(filePath, "/");
    if (sourceDirs != null) {
      for (String sourceDir : sourceDirs) {
        String source = sourceDir.replace('\\', '/');
        if (packageName.startsWith(source) || packageName.contains(source)) {
          if (!source.endsWith("/")) {
            source += "/";
          }
          packageName = StringUtils.substringAfter(packageName, source).replace('/', '.');
          packageName = packageName.replace('/', '.');

          cumulateClassMeasures(packageName, fileName, duplication, fileContainer);
        }
      }
    }
  }

  private void cumulateClassMeasures(String packageName, String fileName, Element duplication, Map<String, ClassTempData> fileContainer) throws ParseException {
    ClassTempData data = fileContainer.get(packageName + fileName);
    if (data == null) {
      data = new ClassTempData(packageName, fileName);
      fileContainer.put(packageName + fileName, data);
    }

    data.cumulate(1, parseNumber(duplication.getAttribute("lines")), parseNumber(duplication.getAttribute("tokens")));
  }

  private static class ClassTempData {
    protected double duplication;
    protected double duplicatedLines;
    protected double duplicatedTokens;
    protected String namespace;
    protected String filename;

    private ClassTempData(String namespace, String filename) {
      this.namespace = namespace;
      this.filename = filename;
    }

    protected void cumulate(double duplication, double duplicatedLines, double duplicatedTokens) {
      this.duplication += duplication;
      this.duplicatedLines += duplicatedLines;
      this.duplicatedTokens += duplicatedTokens;
    }

    protected void saveUsing(MeasuresRecorder recorder) {
      //recorder.createClassMeasure(namespace, filename, false, new MeasureKey(MetricsRepository.DUPLICATIONS), duplication);
      recorder.createClassMeasure(namespace, filename, false, new MeasureKey(MetricsRepository.DUPLICATED_LINES), duplicatedLines);
      //recorder.createClassMeasure(namespace, filename, false, new MeasureKey(MetricsRepository.DUPLICATED_TOKENS), duplicatedTokens);
    }
  }
}
