/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.javancss;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.project.MavenProject;
import org.sonar.commons.MetricsRepository;
import org.sonar.maven.*;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.File;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

public class JavaNcssCollector extends AbstractCollector {
  private XmlReportParser parser;
  private MeasuresRecorder recorder;

  public JavaNcssCollector(File xmlFile, MeasuresRecorder recorder) throws MissingDataException {
    if (xmlFile != null && xmlFile.exists()) {
      parser = loadAndParseXmlReport(xmlFile);
      this.recorder = recorder;
    }
  }

  public JavaNcssCollector(MavenProject project, MeasuresRecorder recorder) throws MissingDataException {
    this(new File(project.getBuild().getDirectory(), "javancss-raw-report.xml"), recorder);
  }

  public void collect() {
    try {
      if (parser != null) {
        collectPackagesCount();
        collectPackageComments();
        collectFileMeasures();
      }

    } catch (ParseException e) {
      throw new XmlParserException(e);

    }
  }

  private void collectPackageComments() throws ParseException {
    NodeList packages = parser.executeXPathNodeList("/javancss/packages/package");
    if (packages != null) {
      for (int i = 0; i < packages.getLength(); i++) {
        Element element = (Element)packages.item(i);
        String packageName = parser.getChildElementValue(element, "name");
        if (".".equals(packageName)) {
          packageName = null;
        }

        double lines = parseNumber(parser.getChildElementValue(element, "javadoc_lines")) +
            parseNumber(parser.getChildElementValue(element, "single_comment_lines")) +
            parseNumber(parser.getChildElementValue(element, "multi_comment_lines"));
        recorder.createPackageMeasure(packageName, new MeasureKey(MetricsRepository.COMMENT_LINES), lines);
      }
    }
  }

  private void collectPackagesCount() throws ParseException {
    double packageCount = parseNumber(parser.executeXPath("count(/javancss/packages/package)"));
    recorder.createProjectMeasure(new MeasureKey(MetricsRepository.PACKAGES_COUNT), packageCount);
  }

  private void collectFileMeasures() throws ParseException {
    NodeList classes = parser.executeXPathNodeList("/javancss/objects/object");
    if (classes != null) {
      for (int i = 0; i < classes.getLength(); i++) {
        Element element = (Element) classes.item(i);
        String fullName = parser.getChildElementValue(element, "name");
        String[] packageAndClass = parsePackageAndClass(fullName);

        createClassMeasure(packageAndClass[0], packageAndClass[1], false, MetricsRepository.NCSS, element, "ncss");
        createClassMeasure(packageAndClass[0], packageAndClass[1], false, MetricsRepository.FUNCTIONS_COUNT, element, "functions");
        recorder.createClassMeasure(packageAndClass[0], packageAndClass[1], false, new MeasureKey(MetricsRepository.CLASSES_COUNT), 1.0);
      }
    }
    createComplexityClassMeasures();

  }

  protected String[] parsePackageAndClass(String fullname) {
    String[] result = new String[2];
    result[0] = null;
    result[1] = fullname;
    if (fullname.indexOf(".") >= 0) {
      result[0] = StringUtils.substringBeforeLast(fullname, ".");
      result[1] = StringUtils.substringAfterLast(fullname, ".");
    }
    return result;
  }

  private void createClassMeasure(String packageName, String className, boolean unitTest, Metric metric, Element classElement, String key) throws ParseException {
    Element element = parser.getChildElement(classElement, key);
    if (element != null && element.getFirstChild() != null && element.getFirstChild().getNodeValue() != null) {
      String value = element.getFirstChild().getNodeValue();
      recorder.createClassMeasure(packageName, className, unitTest, new MeasureKey(metric), parseNumber(value));
    }
  }


  private void createComplexityClassMeasures()
      throws ParseException {

    NodeList functions = parser.executeXPathNodeList("/javancss/functions/function");
    Map<String, Integer> ccnCountperClass = new HashMap<String, Integer>();
    if (functions != null) {
      for (int i = 0; i < functions.getLength(); i++) {
        Element element = (Element) functions.item(i);
        String fullFunctionName = parser.getChildElementValue(element, "name");
        Integer ccnForFunction = new Integer(parser.getChildElementValue(element, "ccn"));
        String packageAndClassName = getPackageAndClassFromFunction(fullFunctionName);
        Integer ccnSum = ccnCountperClass.get(packageAndClassName);
        if (ccnSum == null) {
          ccnSum = 0;
        }
        ccnSum += ccnForFunction;
        ccnCountperClass.put(packageAndClassName, ccnSum);
      }
    }
    for (String fullname : ccnCountperClass.keySet()) {
      Integer ccnForClass = ccnCountperClass.get(fullname);
      String[] packageAndClass = parsePackageAndClass(fullname);
      recorder.createClassMeasure(packageAndClass[0], packageAndClass[1], false, new MeasureKey(MetricsRepository.COMPLEXITY_TOTAL), ccnForClass.doubleValue());
    }
  }

  protected String getPackageAndClassFromFunction(String functionName) {
    String className = StringUtils.substringBeforeLast(functionName, ".");
    return getClassName(className);
  }

  private String getClassName(String classNameCandidate) {
    int lastPointIndex = classNameCandidate.lastIndexOf(".");
    if (lastPointIndex == -1) {
      return classNameCandidate;
    }
    String packageName = StringUtils.substringBeforeLast(classNameCandidate, ".");
    String className = StringUtils.substringAfterLast(classNameCandidate, ".");
    String[] packages = StringUtils.split(packageName, '.');
    for (String pack : packages) {
      if (!isPackageName(pack)) {
        return getClassName(packageName);
      }
    }
    return packageName + "." + className;
  }

  protected boolean isPackageName(String classNameCandidate) {
    if (!Character.isLowerCase(classNameCandidate.charAt(0))) {
      return false;
    }
    for (int i = 0; i < classNameCandidate.length(); i++) {
      char currentLetter = classNameCandidate.charAt(i);
      if (!(Character.isLetterOrDigit(currentLetter) || (currentLetter == '_'))) {
        return false;
      }
    }
    return true;
  }
}

