/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven.surefire;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.project.MavenProject;
import org.sonar.maven.MeasuresRecorder;
import org.sonar.commons.MetricsRepository;
import org.sonar.maven.AbstractCollector;
import org.sonar.maven.XmlParserException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.File;
import java.io.FilenameFilter;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

public class SurefireCollector extends AbstractCollector {
  private MeasuresRecorder recorder;
  private File[] reports;
  private List<UnitTestReport> unitTestReports;

  public SurefireCollector(MavenProject project, MeasuresRecorder recorder) {
    File baseDir = new File(project.getBuild().getDirectory() + "/surefire-reports");
    reports = findSureFireReportFiles(baseDir);
    unitTestReports = new ArrayList<UnitTestReport>();
    this.recorder = recorder;
  }

  public SurefireCollector(File sureFireOutputDirectory, MeasuresRecorder recorder) {
    reports = findSureFireReportFiles(sureFireOutputDirectory);
    unitTestReports = new ArrayList<UnitTestReport>();
    this.recorder = recorder;
  }

  public void collect() {
    if (reports != null) {
      collectFileMeasures();
    }
  }

  public void collectFileMeasures() {
    try {
      DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
      for (File report : reports) {
        Document document = builder.parse(report);
        XPath xpath = XPathFactory.newInstance().newXPath();
        UnitTestReport fileReport = parseReport(xpath, document);
        unitTestReports.add(fileReport);

        saveClassMeasure(fileReport, MetricsRepository.SUREFIRE_SKIPPED, fileReport.skipped);
        saveClassMeasure(fileReport, MetricsRepository.SUREFIRE_TESTS, fileReport.tests);
        saveClassMeasure(fileReport, MetricsRepository.SUREFIRE_ERRORS, fileReport.errors);
        saveClassMeasure(fileReport, MetricsRepository.SUREFIRE_FAILURES, fileReport.failures);
        if (!Double.isNaN(fileReport.time)) {
          saveClassMeasure(fileReport, MetricsRepository.SUREFIRE_TIME, getMeasureTime(fileReport));
        }
      }
    } catch (Exception e) {
      throw new XmlParserException(e);
    }
  }

  private void saveClassMeasure(UnitTestReport fileReport, Metric metric, double value) {
    if (!Double.isNaN(value)) {
      recorder.createClassMeasure(fileReport.packageName, fileReport.className, true, new MeasureKey(metric), value);
    }
  }

  private double getMeasureTime(UnitTestReport fileReport) {
    return scaleValue(fileReport.time * 1000, 3);
  }

  private UnitTestReport parseReport(XPath xpath, Node root) throws XPathExpressionException, ParseException {
    Node testSuite = (Node) xpath.evaluate("//testsuite", root, XPathConstants.NODE);
    UnitTestReport test = new UnitTestReport();
    String fullname = xpath.evaluate("@name", testSuite);
    test.packageName = StringUtils.substringBeforeLast(fullname, ".");
    test.className = StringUtils.substringAfterLast(fullname, ".");
    test.errors = parseNumber(xpath.evaluate("@errors", testSuite));
    test.skipped = parseNumber(xpath.evaluate("@skipped", testSuite));
    test.tests = parseNumber(xpath.evaluate("@tests", testSuite));
    test.time = parseNumber(xpath.evaluate("@time", testSuite));
    test.failures = parseNumber(xpath.evaluate("@failures", testSuite));
    return test;
  }

  private File[] findSureFireReportFiles(File sureFireOutputDirectory) {
    if (sureFireOutputDirectory == null || !sureFireOutputDirectory.exists()) {
      return null;
    }

    return sureFireOutputDirectory.listFiles(new FilenameFilter() {
      public boolean accept(File dir, String name) {
        return name.startsWith("TEST-") && name.endsWith(".xml");
      }
    });
  }

  private class UnitTestReport {
    private String packageName;
    private String className;
    private double errors;
    private double skipped;
    private double tests;
    private double time;
    private double failures;
  }
}